from .base import BaseWrapper
from .product import Product
from functools import cached_property

class Shop(BaseWrapper):
    def __init__(self, link=None, username=None):
        super().__init__()
        self.link = link
        self.username = username
        self.base_url = 'https://shopee.co.id/api/v4/shop/get_shop_detail'
        self.products_url = 'https://shopee.co.id/api/v4/shop/search_items'
        
        if link:
            self.__get_shop_username_from_link()
            self.__get_shop_info()
        elif username:
            self.__get_shop_info()
        else:
            raise ValueError('Either link or username is required')

    def __get_shop_username_from_link(self):
        self.username = self.link.split('/')[-1].split('.')[0]
        return self.username

    def __get_shop_info(self):
        url = f'https://shopee.co.id/api/v4/shop/get_shop_detail?&username={self.username}'
        self.connection.request('GET', url, headers=self.headers)

        with self.connection.getresponse() as response:
            self.data = self.to_json(response)
            self.status_code = response.getcode()
            self.error = self.data['error'] if self.data['error'] else None
            self.shop_id = self.data['data']['shopid'] if self.data['data'] else None

        return self.data

    @cached_property
    def products(self):
        self.products_url = f'{self.products_url}?shopid={self.shop_id}&limit={self.serialize["item_count"]}'
        self.connection.request('GET', self.products_url, headers=self.headers)

        with self.connection.getresponse() as response:
            data = self.to_json(response)
        
        if data['items']:
            return [Product(shop_id=item['shopid'], item_id=item['itemid']) for item in data['items']]

        return []

    @cached_property
    def serialize(self):
        if not self.data or self.error:
            return None

        return {
            'meta': {
                'user_id': self.data['data']['userid'],
                'shop_id': self.data['data']['shopid'],
            },
            'name': self.data['data']['name'],
            'description': self.data['data']['description'] if 'description' in self.data['data'] else None,
            'item_count': self.data['data']['item_count'],
            'country': self.data['data']['country'],
            'rating': {
                'average': self.data['data']['rating_star'] if 'rating_star' in self.data['data'] else None,
                'rating_count': self.data['data']['rating_normal'] + self.data['data']['rating_good'] + self.data['data']['rating_bad'],
                'bad_rating_count': self.data['data']['rating_bad'],
                'good_rating_count': self.data['data']['rating_good'],
                'normal_rating_count': self.data['data']['rating_normal'],
            },
            'is_official_shop': self.data['data']['is_official_shop'],
            'is_shopee_verified': self.data['data']['is_shopee_verified'],
            'shop_location': self.data['data']['shop_location'],
            'shop_covers': [{'url': f'{self.image_base_url}/{cover["image_url"]}'} for cover in self.data['data']['shop_covers'] if cover['type'] == 0] if 'shop_covers' in self.data['data'] else None,
        }



from .base import BaseWrapper
from functools import cached_property

class ProductReview(BaseWrapper):
    def __init__(self, link=None, shop_id=None, item_id=None, filter=0, limit=5, offset=0):
        super().__init__()
        self.link = link
        self.shop_id = shop_id
        self.item_id = item_id
        self.filter = filter
        self.limit = limit
        self.offset = offset
        self.base_url = 'https://shopee.co.id/api/v2/item/get_ratings'
        self.image_base_url = 'https://cf.shopee.co.id/file'

        self.data = self.__get_product_review()

    @cached_property
    def url(self):
        if not self.link and not (self.shop_id and self.item_id):
            raise Exception('link or shop_id and item_id is required')

        if self.link and not (self.shop_id and self.item_id):
            self.shop_id = self.__get_shop_id()
            self.item_id = self.__get_item_id()

        return f'{self.base_url}?itemid={self.item_id}&shopid={self.shop_id}&filter={self.filter}&limit={self.limit}&offset={self.offset}&type=0'

    def __get_product_review(self):
        self.connection.request('GET', self.url, headers=self.headers)

        with self.connection.getresponse() as response:
            self.data = self.to_json(response)
            self.status_code = response.getcode()
            self.error = self.data['error'] if self.data['error'] else None

        return self.data

    def __serialize_review(self):
        response = []
        for rating in self.data['data']['ratings']:
            rating_data = {
                'item_id': rating['itemid'],
                'user_id': rating['userid'],
                'username': rating['author_username'],
                'comment': rating['comment'],
                'created_at': rating['ctime'],
                'like_count': rating['like_count'],
                'images': [f'{self.image_base_url}/{image}' for image in rating['images']],
                'videos': [{'cover': video['cover'], 'url': video['url']} for video in rating['videos']],
                'tags': rating['tags'],
                'rating_star': rating['rating_star'],
            }
            response.append(rating_data)
        return response

    @cached_property
    def serialize(self):
        if not self.data or self.error:
            return None

        return {
            'meta': {
                'total': self.data['data']['item_rating_summary']['rating_total'],
                'with_media': self.data['data']['item_rating_summary']['rcount_with_media'],
                'with_context': self.data['data']['item_rating_summary']['rcount_with_context']
            },
            'reviews': self.__serialize_review()
        }



from .base import BaseWrapper
from .review import ProductReview
from functools import cached_property

class Product(BaseWrapper):
    def __init__(self, link = None, shop_id = None, item_id = None, **kwargs):
        super().__init__()
        self.link = link
        self.shop_id = shop_id
        self.item_id = item_id
        self.kwargs = kwargs
        self.base_url = 'https://shopee.co.id/api/v4/item/get'

    @cached_property
    def reviews(self):
        return ProductReview(shop_id=self.shop_id, item_id=self.item_id, **self.kwargs)

    @cached_property
    def url(self):
        if not self.link and not (self.shop_id and self.item_id):
            raise Exception('link or shop_id and item_id is required')

        if self.link and not (self.shop_id and self.item_id):
            self.shop_id = self.__get_shop_id()
            self.item_id = self.__get_item_id()

        return f'{self.base_url}?itemid={self.item_id}&shopid={self.shop_id}'

    def __get_shop_id(self):
        product_slug = str(self.link).split('/')[-1]
        return product_slug.split('.')[1]

    def __get_item_id(self):
        product_slug = str(self.link).split('/')[-1].split('?')[0]
        return product_slug.split('.')[2]
    
    def __get_product_info(self):
        self.connection.request('GET', self.url, headers=self.headers)

        with self.connection.getresponse() as response:
            self.data = self.to_json(response)
            self.status_code = response.getcode()
            self.error = self.data['error'] if self.data['error'] else None

        return self.data

    def __serialize_variation_options(self, variation):
        if not self.data or self.error:
            return None

        result = []

        for index, option in enumerate(variation['options']):
            result.append({
                'name': option,
                'image': f'{self.image_base_url}/{variation["images"][index]}' if variation['images'] != None else None,
            })

        return result

    def __serialize_variations(self):
        if not self.data or self.error:
            return None

        result = []

        for variation in self.data['data']['tier_variations']:
            result.append({
                'name': variation['name'],
                'options': self.__serialize_variation_options(variation)
            })

        return result

    @cached_property
    def serialize(self):
        self.__get_product_info()
        if not self.data or self.error:
            return None

        return {
                'meta': {
                    'item_id': self.data['data']['itemid'],
                    'shop_id': self.data['data']['shopid'],
                    'user_id': self.data['data']['userid'],
                    'shopee_endpoint': self.url
                },
                'product': {
                    'name': self.data['data']['name'],
                    'description': self.data['data']['description'],
                    'brand': self.data['data']['brand'],
                    'price_min': self.data['data']['price_min'],
                    'price_max': self.data['data']['price_max'],
                    'price': self.data['data']['price'],
                    'discount': self.data['data']['discount'],
                    'sold': self.data['data']['sold'] if self.data['data']['sold'] > self.data['data']['historical_sold'] else self.data['data']['historical_sold'],
                    'stock': self.data['data']['stock'],
                    'like': self.data['data']['liked_count'],
                    'rating': {
                        'average': self.data['data']['item_rating']['rating_star'],
                        'count': self.data['data']['item_rating']['rating_count'][0],
                        'star': {
                            '1': self.data['data']['item_rating']['rating_count'][1],
                            '2': self.data['data']['item_rating']['rating_count'][2],
                            '3': self.data['data']['item_rating']['rating_count'][3],
                            '4': self.data['data']['item_rating']['rating_count'][4],
                            '5': self.data['data']['item_rating']['rating_count'][5],
                        }
                    },
                    'variations': self.__serialize_variations(),
                    'models': [
                        {
                            'item_id': model['itemid'],
                            'model_id': model['modelid'],
                            'tier_index': model['extinfo']['tier_index'],
                            'name': model['name'],
                            'price': model['price'],
                            'stock': model['stock']
                        } for model in self.data['data']['models']],
                    'attributes': self.data['data']['attributes'],
                    'categories': self.data['data']['categories'],
                    'images': [f'{self.image_base_url}/{image}' for image in self.data['data']['images']],
                    'videos': [
                        {
                            'id': video['video_id'],
                            'thumbnail': f'{self.image_base_url}/{video["thumb_url"]}',
                            'url': video['default_format']['url'],
                            'format': video['default_format']['profile'],
                            'definition': video['default_format']['defn']
                        } for video in self.data['data']['video_info_list']] if self.data['data']['video_info_list'] else None,
                }
            }


"""
Discord API Wrapper
~~~~~~~~~~~~~~~~~~~

A basic wrapper for the Discord API.

:copyright: (c) 2015-present Rapptz
:license: MIT, see LICENSE for more details.

"""

__title__ = 'shopee-wrapper'
__author__ = 'munawariz'
__license__ = 'MIT'
__copyright__ = 'Copyright 2022-present munawariz'
__version__ = '0.3'

from .base import *
from .product import *
from .review import *
from .shop import *


import http.client
import json

class BaseWrapper:
    base_url = 'https://shopee.co.id'
    image_base_url = 'https://cf.shopee.co.id/file'
    video_base_url = 'http://play-ws.vod.shopee'
    status_code = 200
    error = None

    def __init__(self):
        self.establish_connection()
        self.generate_headers()

    def establish_connection(self):
        self.connection = http.client.HTTPSConnection('shopee.co.id')

        return self.connection

    def generate_headers(self):
        self.headers = {
            'Cookie': 'REC_T_ID=6bab0c7a-f8f1-11ec-a247-2cea7f4774b9; SPC_F=zz3O96ULg4nQMj2PurlTBZecedxhewhg; SPC_R_T_ID=381h9C7xlIhz0TP9YSDXmro/WaRDXA5bSVV704I/evi5gR4rxN9KLJpXGK6DJ7LgvISvN3uQyKyDiuq8Fs0TcNAhGC56FtL6KDa4dXJecY8tfxlZ/AtC85SjcMjjpflEPPv92h+oH+XtxHnWlzDvSUbXIsk+s/WXXJbVJVTSrjo=; SPC_R_T_IV=N3ppYVBuYWxaWDNJZmtkVQ==; SPC_SI=rBm8YgAAAABHUGV1bDBib2jXBQAAAAAAd0JzaXU4QTA=; SPC_T_ID=381h9C7xlIhz0TP9YSDXmro/WaRDXA5bSVV704I/evi5gR4rxN9KLJpXGK6DJ7LgvISvN3uQyKyDiuq8Fs0TcNAhGC56FtL6KDa4dXJecY8tfxlZ/AtC85SjcMjjpflEPPv92h+oH+XtxHnWlzDvSUbXIsk+s/WXXJbVJVTSrjo=; SPC_T_IV=N3ppYVBuYWxaWDNJZmtkVQ==',
            'Connection': 'close',
            'AF-AC-ENC-DAT': 'AAcyLjUuMi0yAAABhdbDaGQAAAzeAoAAAAAAAAAAAiS4mTYXtBIM3/aVutv7vv95BRd698jZ/901tpYwiYTjGF/2xGLP3PvqXW8MCO4GZjd0LOP5ucxzA6P+HlxI8a0e0EITl/82EyfDx/bVRcPaaRvYm0iAHc9YPNN3N+N4xKMBpujmZ7VFnAs5HbXqfjiMTh5Wl/82EyfDx/bVRcPaaRvYm7Q7Y7iER0YZDuPlzDCijEDdpMsZ74BuVdOliab30GDtvsGH5bvVcZqN1VGQ/nuCwUpbg/iduB+irk6FJ69PY+1fOBbJvLvYAVLMa+wqKcDPO/6faKm+ZVKsH4uNXJ5E7ppLBWCjreCQPoeLs2qGai1PXjEGGWy3DMVVukZKDs+2mB2ojoppDRDE4m+SQNZ5dTPdxAi7ISkMhAiEVo9TfIv6D60EAfL6N+xglYXjLTEDPHwai1Hpp02msuwKPRyHq7M79uFmkEeSIzznOECBsA1WP5bOtSjQMliLPHU49jrTIPznogusS6HG56/Ud4S2QU0ah2EGVFJ6qY6RA65n+hoQ93OHsMtIrOd1VgYAe4oU/t4Mua5+k/FLRe4WuevGICG019hD/ZVAy6L8CRUdBtFHZYe7jrVGRDP+TMtRKgf3r9BOlMHgrMoCBnXxFjDEyCgFa119bbLTGYa9RFB4bXC1BnYf0sH6Fd3QTyiHd8Keybp71S0T2mIq5wg4Hh3ayt22wZu1fv5e3dGER2nDn0Gy4FawdTAEX7UzbO9sQyW4bh5jEx88pTZkjBaE0hpNb7DrFa8fe4xhhZ0urMpOtyzV+h2R+es0bAdJ0P1V6oB+MKkk05uyBi99RpxpITDk9Nrht2CSin9letnhWPkJTMuoAJjhjlGFNhnzHRlHxeSi',
            'User-Agent': 'Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/108.0.0.0 Safari/537.36'
        }

        return self.headers

    @staticmethod
    def to_json(response):
        data = response.read()

        return json.loads(data.decode('utf-8'))

    def serialize(self):
        raise NotImplementedError




import pathlib
from setuptools import setup

HERE = pathlib.Path(__file__).parent
README = (HERE / "README.md").read_text()
setup(
  name="shopee_wrapper",
  version="0.3",
  description="Library that wrap shopee.co.id api using http.client",
  long_description=README,
  long_description_content_type="text/markdown",
  author="Hariz Sufyan Munawar",
  author_email="munawarhariz@gmail.com",
  license="Apache License",
  packages=["shopee_wrapper"],
  zip_safe=False,
)